% P = subspaceMatching(W1,W2,auxFeatures,Ln)
% 
% This function computes the permutation matrix P such that
% 
%               P* = argmin_P ||null(W1)*P*W2'||_Ln
%                       s.t.   P is a doubly-stochastic matrix
%
%
% Input arguments:
% 
%     W1 - r-rank subspace basis
%                 
%     W2 - k-rank subspace basis
%     (r must be greater or equal than k)
%     
%     auxFeatures - number of artificial features
%
%     Ln - The used norm 
%     (This argument should be 1, 2 or 'Inf')
%
% 
% For details, see:
% 
%    Marques, M., Stoic, M. and Costeira, J.. "Subspace matching: Unique solutions to point matching with geometric constraints",
%    ICCV2009 - 12th IEEE International Conference on Computer Vision, Kyoto, Japan, September 2009.
            
function P = subspaceMatching(W1,W2,auxFeatures,Ln)

W1 = W1 - mean(W1,2)*ones(1,size(W1,2));
W2 = W2 - mean(W2,2)*ones(1,size(W2,2)); 

[U,D,V1] = svd(W1,'econ'); V1 = V1(:,1:4);

P = sdpvar(size(W1,2),size(W2,2),'full');

constraints = [sum(P,1) <= 1, sum(P,2) <= 1, P(1:(auxFeatures+1),1:(auxFeatures+1)) == eye(auxFeatures+1), P(:) >= 0];
costFunction = norm(null(V1')'*P*W2',Ln);
solvesdp(constraints,costFunction)
P = double(P(auxFeatures+1:end,auxFeatures+1:end));
